import { join, map } from "lodash/fp"
import {
  getLock,
  releaseLock,
  $,
  isNolock,
  apiWrapper,
  events,
} from "../common"
import { ACCESS_LEVELS_LOCK_FILE, ACCESS_LEVELS_FILE } from "./authCommon"
import { validateAccessLevels } from "./validateAccessLevels"
import { permission } from "./permissions"

export const saveAccessLevels = app => async accessLevels =>
  apiWrapper(
    app,
    events.authApi.saveAccessLevels,
    permission.writeAccessLevels.isAuthorized,
    { accessLevels },
    _saveAccessLevels,
    app,
    accessLevels
  )

export const _saveAccessLevels = async (app, accessLevels) => {
  const validationErrors = validateAccessLevels(app)(accessLevels.levels)
  if (validationErrors.length > 0) {
    const errs = $(validationErrors, [map(e => e.error), join(", ")])
    throw new Error(`Access Levels Invalid: ${errs}`)
  }

  const lock = await getLock(app, ACCESS_LEVELS_LOCK_FILE, 2000, 2)

  if (isNolock(lock)) {
    throw new Error("Could not get lock to save access levels")
  }

  try {
    const existing = await app.datastore.loadJson(ACCESS_LEVELS_FILE)
    if (existing.version !== accessLevels.version) {
      throw new Error(
        "Access levels have already been updated, since you loaded"
      )
    }

    accessLevels.version++

    app.datastore.updateJson(ACCESS_LEVELS_FILE, accessLevels)
  } finally {
    await releaseLock(app, lock)
  }
}
