import { generate } from "shortid"
import { joinKey } from "../common"
import { getLastPartInKey } from "../templateApi/hierarchy"
import {
  IndexNodeKeyFolder,
  BUILDINDEX_BATCH_COUNT,
  IndexNodeKeyBatchFolder,
  TRANSACTIONS_FOLDER,
  getTransactionId,
  CREATE_RECORD_TRANSACTION,
  UPDATE_RECORD_TRANSACTION,
  DELETE_RECORD_TRANSACTION,
  BUILD_INDEX_TRANSACTION,
} from "./transactionsCommon"

export const transactionForCreateRecord = async (app, record) =>
  await transaction(
    app.datastore,
    CREATE_RECORD_TRANSACTION,
    record.key,
    { record },
    getTransactionKey_Records
  )

export const transactionForUpdateRecord = async (app, oldRecord, newRecord) =>
  await transaction(
    app.datastore,
    UPDATE_RECORD_TRANSACTION,
    newRecord.key,
    { oldRecord, record: newRecord },
    getTransactionKey_Records
  )

export const transactionForDeleteRecord = async (app, record) =>
  await transaction(
    app.datastore,
    DELETE_RECORD_TRANSACTION,
    record.key,
    { record },
    getTransactionKey_Records
  )

export const transactionForBuildIndex = async (
  app,
  indexNodeKey,
  recordKey,
  count
) => {
  const transactionFolder = IndexNodeKeyBatchFolder(indexNodeKey, count)
  if (count % BUILDINDEX_BATCH_COUNT === 0) {
    await app.datastore.createFolder(transactionFolder)
  }

  return await transaction(
    app.datastore,
    BUILD_INDEX_TRANSACTION,
    recordKey,
    { recordKey },
    id => joinKey(transactionFolder, id)
  )
}

export const createBuildIndexFolder = async (datastore, indexNodeKey) =>
  await datastore.createFolder(IndexNodeKeyFolder(indexNodeKey))

const getTransactionKey_Records = id => joinKey(TRANSACTIONS_FOLDER, id)

const transaction = async (
  datastore,
  transactionType,
  recordKey,
  data,
  getTransactionKey
) => {
  const recordId = getLastPartInKey(recordKey)
  const uniqueId = generate()
  const id = getTransactionId(recordId, transactionType, uniqueId)

  const key = getTransactionKey(id)

  const trans = {
    transactionType,
    recordKey,
    ...data,
    id,
  }

  await datastore.createJson(key, trans)

  return trans
}
